#!/usr/local/bin/perl -w
# 
# Writer:         wangxy <wxyang1988@126.com>
# Program Date:   2011.
# Modifier:       wangxy <wxyang1988@126.com>
# Last Modified:  2012-1-5
# This program adds taxonomic information from the SAP html files to an OTU table.
##########################################################

my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#########################################################

my %opts;
GetOptions(\%opts,"i1=s","i2=s","p=s","o=s","h" ); 

#&help()if(defined $opts{h});
if(!defined($opts{i1}) || !defined($opts{i2}) || !defined($opts{p}) || !defined($opts{o}) ||  defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver
		
		This program extracts the taxonomic information from the SAP output
		and adds it to the -i1 otu table.
		
		Each SAP output includes the folder "html"
		
		If there are n SAP runs to process, number the html folders from 
		html.1 to html.n and place all in a single folder, designated by -i2
		
		SAP assigns taxonomies at different posterior probabilities. Use -p to
		designate the lowest allowable probability (e.g. 80% = -p 80)
		
		If a sequence has no taxonomic information, NO_TAXONOMY is written to the
		table.
		
		The output file -o is the new name of the otu table with taxonomy added.

	Usage:

		-i1     otu table                                   must be given

		-i2     folder containing html.1 to html.n          must be given

		-p      lowest SAP posterior percentage allowed     must be given

		-o      output file                                 must be given

		-h     Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
print "\nStart Time :[$Time_Start]\n\n";
################
my $programe_dir=basename($0);
my $in1=$opts{i1};
my $in2=$opts{i2};
my $percentage=$opts{p};
my $out=$opts{o};

#taxonomy_SAP_80%_probability, add to the first line like this " taxonomy_SAP_$percentage%_probability"

my %otutable;
my %otu_id;

my $line1;
my $line2;

open OUT,">$out" ||die "~~~~~~~~~~~~\n";
open IN,"$in1" || die "~~~~~~~~~~~~~\n";
$line1=<IN>;
chomp($line1);
$line2=<IN>;
chomp ($line2);
while (<IN>) 
{
	chomp;
	next if (/^\s*$/);
	my @tmp=split/\t/,$_;
	$otutable{$tmp[0]}=$_;
}
close IN;

print "$line1\n";
print "$line2\n";

my @file1=glob"$in2/html.*";
for (my $i=0;$i<@file1;$i++)
{
	my @file2=glob"$file1[$i]/clones/*.html";
	for (my $j=0;$j<@file2;$j++)
	{
		my $otuid;
		open IN2,"$file2[$j]"|| die "~~~~~~~~~~~\n";
		while (<IN2>)
		{
			
			my $perc;
			my $percent;
			if (/^<h1>/)
			{
				my $pre_otuid=(split/</,$_)[1];
				$otuid=(split/\s+/,$pre_otuid)[1];
			}
			if (/\s+\+/)
			{
				if (/\(superkingdom\)/) {
					my $reotu=(split/>/,$_)[1];
					my $otu=(split/</,$reotu)[0];
					$otu=~s/\s+/_/g;
					$perc=(split/\) /,$_)[-1];
					if (/~0/){$percent=0;}else{$percent=(split/%/,$perc)[0];}
					if ($percent>=$percentage)
					{
						$otu_id{$otuid}.="superkingdom $otu; ";
					}
				}
				if (/\(phylum\)/) {
					my $reotu=(split/>/,$_)[1];
					my $otu=(split/</,$reotu)[0];
					$otu=~s/\s+/_/g;
					$perc=(split/\) /,$_)[-1];
					if (/~0/){$percent=0;}else{$percent=(split/%/,$perc)[0];}
					if ($percent>=$percentage)
					{
						$otu_id{$otuid}.="phylum $otu; ";
					}
				}
				if (/\(class\)/) {
					my $reotu=(split/>/,$_)[1];
					my $otu=(split/</,$reotu)[0];
					$otu=~s/\s+/_/g;
					$perc=(split/\) /,$_)[-1];
					if (/~0/){$percent=0;}else{$percent=(split/%/,$perc)[0];}
					if ($percent>=$percentage)
					{
						$otu_id{$otuid}.="class $otu; ";
					}
				}
				if (/\(order\)/) {
					my $reotu=(split/>/,$_)[1];
					my $otu=(split/</,$reotu)[0];
					$otu=~s/\s+/_/g;
					$perc=(split/\) /,$_)[-1];
					if (/~0/){$percent=0;}else{$percent=(split/%/,$perc)[0];}
					if ($percent>=$percentage)
					{
						$otu_id{$otuid}.="order $otu; ";
					}
				}
				if (/\(family\)/) {
					my $reotu=(split/>/,$_)[1];
					my $otu=(split/</,$reotu)[0];
					$otu=~s/\s+/_/g;
					$perc=(split/\) /,$_)[-1];
					if (/~0/){$percent=0;}else{$percent=(split/%/,$perc)[0];}
					if ($percent>=$percentage)
					{
						$otu_id{$otuid}.="family $otu; ";
					}
				}
				if (/\(genus\)/) {
					my $reotu=(split/>/,$_)[1];
					my $otu=(split/</,$reotu)[0];
					$otu=~s/\s+/_/g;
					$perc=(split/\) /,$_)[-1];
					if (/~0/){$percent=0;}else{$percent=(split/%/,$perc)[0];}
					if ($percent>=$percentage)
					{
						$otu_id{$otuid}.="genus $otu; ";
					}
				}
				if (/\(species\)/) {
					my $reotu=(split/>/,$_)[1];
					my $otu=(split/</,$reotu)[0];
					$otu=~s/\s+/_/g;
					$perc=(split/\) /,$_)[-1];
					if (/~0/){$percent=0;}else{$percent=(split/%/,$perc)[0];}
					if ($percent>=$percentage)
					{
						$otu_id{$otuid}.="species $otu; ";
					}
				}

			}
		
		}
	close IN2;
	}
}


my %otu_id2;
foreach my $k (keys %otu_id) 
{
	my @a=split/\s+/,$otu_id{$k};
	if ($otu_id{$k}=~m /species/) {
		next if (!defined $a[6]);
		if ($a[6] eq "species") {$otu_id2{$k}="$a[1]$a[3]$a[5]others;others;others;$a[7]";}
		next if (!defined $a[8]);
		if ($a[8] eq "species") {$otu_id2{$k}="$a[1]$a[3]$a[5]$a[7]others;others;$a[9]";}
		next if (!defined $a[10]);
		if ($a[10] eq "species") {$otu_id2{$k}="$a[1]$a[3]$a[5]$a[7]$a[9]others;$a[11]";}
		else {$otu_id2{$k}="$a[1]$a[3]$a[5]$a[7]$a[9]$a[11]$a[13]";}
	}
	else 
	{
		for (my $i=1;$i<@a ;$i+=2) 
		{
			$otu_id2{$k}.=$a[$i];
		}
	}

}

print OUT $line1,", taxonomy_SAP_",$percentage,"%_probability\n";
print OUT $line2,"\tConsensus Lineage\n";

foreach my $s(keys %otutable)
{
	if (exists $otu_id2{$s}) 
	{
		$otu_id2{$s}=~s/\;$//g;
		print OUT "$otutable{$s}\t$otu_id2{$s}\n";
	}
	else
	{
		print OUT "$otutable{$s}\tNO_TAXONOMY\n";
	}
}

close OUT;

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
print "\nEnd Time :[$Time_End]\n\n";

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}
